*******************************
*
* BENCHMARK_VAR.PRG
* THIS RATS PROGRAM WAS WRITTEN BY MATTEO IACOVIELLO FOR RATS
*
* RATS PROGRAM NEEDED TO:
* 1) GENERATE AND STORE THE IMPULSE RESPONSES AND STANDARD ERRORS
* OF THE VAR OF FIGURE 1 IN THE PAPER
* 2) ESTIMATE THE TAYLOR RULE OF THE "ECONOMETRIC METHODOLOGY SECTION"
* OF THE PAPER
*
* @MATTEO IACOVIELLO, IACOVIEL@BC.EDU
*
* USES SECTION OF CODE FROM THE PROGRAM MONTEVAR.PRG AS DESCRIBED IN
* RATS.5 USER GUIDE, ON PAGE 398 AND 399
* MINOR CHANGES ARE IN BETWEEN THE LINES WITH THE WORDS "MATTEO"
*
* MANY OF THE DATA SERIES ARE NOT USED IN THE FINAL SPECIFICATION,
* BUT ARE KEPT IN THE FILES
*
***********************************************

**********************************
* CLEAR MEMORY
**********************************

END XXX


***********************************************
*
* DOWNLOADING DATA FROM EXCEL FILES AND STORE THEM IN A .RAT FILE
* FOR CONVENIENCE
* DATA SOURCES ARE DEFINITIONS ARE CONTAINED IN THE EXCEL FILES
*
************************************************

CALENDAR 1969 1 4; ALL 2003:4 ;
OPEN DATA C:\E\HOUSEPRICES_AER\VAR_AER\US_QUARTERLY_DATA.XLS;
DATA(FOR=EXCEL, ORG=OBS)
DEDIT C:\E\HOUSEPRICES_AER\VAR_AER\USDATA.RAT
STORE GDPPOT GDP HPI GCEC1 GNP GDPDEF GNPDEF GPBUQF PCE GPDIC1 NCREIF OFHEO
SAVE; END 1;

CALENDAR 1947 1 4; ALL 2003:12 ;
OPEN DATA C:\E\HOUSEPRICES_AER\VAR_AER\US_MONTHLY_DATA.XLS;
DATA(FOR=EXCEL, ORG=OBS, COMPACT=FIRST)
DEDIT C:\E\HOUSEPRICES_AER\VAR_AER\USDATA.RAT
SET FEDBEGIN = FED
STORE FEDBEGIN PSCCOM CRBSPOT
SAVE; END 1;

CALENDAR 1947 1 12; ALL 2003:12 ;
OPEN DATA C:\E\HOUSEPRICES_AER\VAR_AER\US_MONTHLY_DATA.XLS;
DATA(FOR=EXCEL, ORG=OBS)
DEDIT C:\E\HOUSEPRICES_AER\VAR_AER\USDATA.RAT
STORE CPI CNP16OV PPI FED
SAVE; QUIT; END 1;



CALENDAR 1969 1 4; ALL 2003:4 ;
OPEN DATA C:\E\HOUSEPRICES_AER\VAR_AER\USDATA.RAT;
DATA(FOR=RAT, ORG=OBS)



***********************************************
*
* CREATE FILTERED VARIABLES
*
***********************************************

* BOUNDS FOR BANDPASS FILTER

COMPUTE UPPERO=1
COMPUTE LOWERO=32

* LOAD BANDPASS FILTER SOURCE CODE

SOU C:\E\HOUSEPRICES_AER\VAR_AER\AUXILIARY_FILES\BPFILTER.SRC

SET COSTANTE = 1; ACCUMULATE COSTANTE 1970:1 2003:4 TIME 1970:1
SET TIMESQ = TIME*TIME/1000


******************************
* CREATE DETRENDED GDP
********************************

* GDP LESS GOVT CONSUMPTION
SET GDPPRIV = GDP - GCEC1 ;
LOG GDPPRIV / LGDPPRIV;
DIFF GDPPRIV / DGDPPRIV

* GDP
SET LGDP = LOG(GDP) ;
DIFF LGDP / DLGDP

* GDP, LINEARLY DETRENDED
LINREG(NOPRINT) LGDP / LGDP_LIN;
# CONSTANT TIME

* GDP VERSUS POTENTIAL (CBO ESTIMATE)
SET GDPGAP = GDP/GDPPOT

* GNP
SET LGNP = LOG(GNP)

* CREATE DETRENDED SERIES
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO)  LGDP / LGDP_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO)  LGDPPRIV / LGDPPRIV_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LGNP / LGNP_F




********************************
* CREATE DETRENDED PRICE VARIABLES
************************************

* CONSUMER PRICES
SET LCPI = LOG(CPI) ;  DIFF LCPI / DLCPI
SET LGDPDEF = LOG(GDPDEF) ;  DIFF LGDPDEF / DLGDPDEF
SET LGNPDEF = LOG(GNPDEF) ;  DIFF LGNPDEF / DLGNPDEF
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO)  LGDP / LGDP_F


LINREG(NOPRINT) LGDPDEF / LGDPDEF_LIN; # CONSTANT TIME
DIFF LGDPDEF_LIN / DLGDPDEF_LIN ;
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) DLGDPDEF / DLGDPDEF_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) DLGNPDEF / DLGNPDEF_F


* HOUSE PRICES
SET LHPI =  LOG(HPI) - LOG(GDPDEF);     DIFF LHPI / DLHPI
LINREG(NOPRINT) LHPI / LHPI_LIN; # CONSTANT TIME TIMESQ
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LHPI / LHPI_F

SET LHPI2 =  LOG(NCREIF) - LOG(GDPDEF);     DIFF LHPI2 / DLHPI2
LINREG(NOPRINT) LHPI2 / LHPI2_LIN; # CONSTANT TIME TIMESQ
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LHPI2 / LHPI2_F

SET LHPI3 =  LOG(OFHEO) - LOG(GDPDEF);     DIFF LHPI3 / DLHPI3
LINREG(NOPRINT) LHPI3 / LHPI3_LIN; # CONSTANT TIME TIMESQ
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LHPI3 / LHPI3_F




* INTEREST RATE (FED IS AVG, FEDBEGIN IS BEGIN QRT)
SET FEDBEGIN400 = FEDBEGIN/400
SET FEDAVERAGE400 = FED/400
LINREG(NOPRINT) FEDBEGIN400 / FED400_LIN; # CONSTANT TIME
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) FEDBEGIN400 / FED400_F



**********************************
* CREATE OTHER DETRENDED VARIABLES
*********************************

* CONSUMPTION EXPENDITURE (PRIVATE)
SET LPCE = LOG(PCE)
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LPCE / LPCE_F



* AUXILIARY VARIABLES: COMMODITY PRICES
LOG PSCCOM / LPCOM; DIFF LPCOM / DLPCOM
LOG PPI / LPPI ; DIFF LPPI / DLPPI
LOG CRBSPOT / LCRB ; DIFF LCRB / DLCRB
MVSTATS(SPAN=5,CENTERED,MEANS=DLCRB_SMOOTH) DLCRB

@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LPCOM / LPCOM_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) DLPCOM / DLPCOM_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) LPPI / LPPI_F
@BPFILTER(UPPER=UPPERO,LOWER=LOWERO) DLPPI / DLPPI_F

@BPFILTER(UPPER=6,LOWER=LOWERO) LCRB / LCRB_F
@BPFILTER(UPPER=6,LOWER=LOWERO) DLCRB / DLCRB_F






************************************************************
*
* CREATE VARIOUS DUMMIES FOR SHIFT IN US MONETARY POLICY FROM 1979Q4 TO
* 1982Q4
*
************************************************************

SET D793 = 0; SET D793 1979:3 1979:3 = 1
SET D794 = 0; SET D794 1979:4 1979:4 = 1

SET D801 = 0; SET D801 1980:1 1980:1 = 1
SET D802 = 0; SET D802 1980:2 1980:2 = 1
SET D803 = 0; SET D803 1980:3 1980:3 = 1
SET D804 = 0; SET D804 1980:4 1980:4 = 1

SET D811 = 0; SET D811 1981:1 1981:1 = 1
SET D812 = 0; SET D812 1981:2 1981:2 = 1
SET D813 = 0; SET D813 1981:3 1981:3 = 1
SET D814 = 0; SET D814 1981:4 1981:4 = 1

SET D821 = 0; SET D821 1982:1 1982:1 = 1
SET D822 = 0; SET D822 1982:2 1982:2 = 1
SET D823 = 0; SET D823 1982:3 1982:3 = 1
SET D824 = 0; SET D824 1982:4 1982:4 = 1

SET D794AFTER = 0; SET D794AFTER 1979:4 2003:4 = 1





****************************************************************
*
* CONSTRUCTION OF KEY SERIES
* THESE ARE THE SERIES THAT ENTER THE FINAL SPECIFICATION
*
***************************************************************

SET Y = 100*LGDP_F
SET P = 100*DLGDPDEF
SET Q = 100*LHPI_F
SET R = 100*FEDBEGIN400

* CONSUMPTION SERIES FOR USE IN CONSUMPTION VAR
SET CE = 100*LPCE_F


************************************************
* SAVE ALL THE SERIES IN A SINGLE RATS FILE
* EXTENSION .RAT
************************************************

OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\USDATA_ALL.RAT;
COPY(UNIT=COPY,FORMAT=RATS,ORG=COLUMNS) 1969:1 2003:4
CLOSE COPY








****************************************
* DEFINE SAMPLE PERIOD
****************************************

COMPUTE BEG = 1974:1
COMPUTE ENDE = 2003:2



SMPL BEG ENDE

****************************************
*
* BASELINE REGRESSION FOR TAYLOR RULE
*
*****************************************

LINREG R ;    #  CONSTANT R{1} Y{1} P{1} D794AFTER

COMPUTE FI_P = %BETA(4)/(1-%BETA(2))-1
WRITE 'FI_P' FI_P

COMPUTE FI_Y = %BETA(3)/(1-%BETA(2))
WRITE 'FI_Y' FI_Y


* CHECK THAT IF PLUGGED INTO MODEL THE RULES SATISFIES DETERMINACY
* OF THE RATIONAL EXPECTATION EQUILIBRIUM

COMPUTE DETERMINACY_IF_MORE_THAN_ONE = FI_P + 1 + .1165*FI_Y
WRITE 'DETERMINACY_IF_MORE_THAN_ONE' DETERMINACY_IF_MORE_THAN_ONE




******************************************
*
* CREATE A NICE 2-BY-2 PLOT OF THE TIME SERIES USED
* AND GIVE SOME SUMMARY STATISTICS
*
*********************************************

SPGRAPH(HFIELDS=2, VFIELDS=2)
  GRPARM KEYLABELING 40 AXISLABELING 28
  GRAPH(KEY=UPLEFT) 2;  # Y ; # Q
  GRAPH(KEY=UPLEFT) 1;  # P
  GRAPH(KEY=UPLEFT) 1;  # Q
  GRAPH(KEY=UPLEFT) 1;  # R
SPGRAPH(DONE)
  GRPARM KEYLABELING 20 AXISLABELING 14


TABLE / R Q P Y










***************************************************************
*
* ESTIMATE VAR
*
***************************************************************


COMPUTE NVAR=4			;*NUMBER OF VARIABLES
COMPUTE NSTEPS=21		;*NUMBER OF RESPONSE STEPS
COMPUTE NDRAWS=250		;*NUMBER OF DRAWS
* APPLY CORRECTION FOR NUMBER OF STEPS BASED ON TIME PERIOD
COMPUTE NSTEP = NSTEPS - 1;

COMPUTE LAGS=2			;*NUMBER OF LAGS

SYSTEM(MODEL=VARMODEL)
VARIABLES R P Q Y
LAGS 1 2
KFSET XXX
DET CONSTANT TIME LCRB{1} D794AFTER


END(SYSTEM)
ESTIMATE(COEFFS=TRY,RESIDUALS=RESVAR,OUTSIGMA=VMAT)

COMPUTE VMATCAZ = .5*VMAT+.5*TR(VMAT)
WRITE VMATCAZ








********************************************
* FROM NOW ON, I USE ENTIRE LINES OF CODE FROM
* THE MONTEVAR.PRG FILE FROM THE RATS
* PUBLIC DOMAIN
*********************************************




*******RATS***********************************
DECLARE RECT SXX SVTR SWISH BETAOLS BETADRAW
DECLARE SYMM SIGMAD




COMPUTE SXX    = %DECOMP(XXX)
COMPUTE SVTR   = TR(%DECOMP(VMAT))

COMPUTE BETAOLS=%MODELGETCOEFFS(VARMODEL)
COMPUTE NCOEF  =%ROWS(SXX)
*******RATS***********************************






*******RATS***********************************
DECLARE RECT[VECT] FIRST(NVAR,NVAR) SECOND(NVAR,NVAR) FOURTH(NVAR,NVAR) HSTAR(NVAR,NVAR)
DECLARE RECT[SERIES] IMPULSES(NVAR,NVAR)
DECLARE RECT RANC(NCOEF,NVAR)

DO I=1,NVAR
   DO J=1,NVAR
      DIM FIRST(I,J)(NSTEP) SECOND(I,J)(NSTEP) FOURTH(I,J)(NSTEP) HSTAR(I,J)(NSTEP)
      COMPUTE FIRST(I,J)=%CONST(0.0),SECOND(I,J)=%CONST(0.0),FOURTH(I,J)=%CONST(0.0)
   END DO J
END DO I

/*
IT IS COMPUTED BY POSTMULTIPLYING THE KRONECKER PRODUCT OF THE DECOMPOSITIONS OF SIGMA AND X'X INVERSE BY A VECTOR OF STANDARD NORMAL DRAWS.
WITH ANTITHETIC ACCELERATION, EACH "DRAW" REQUIRES TWO CALCULATIONS OF THE IMPULSE RESPONSE FUNCTION. TO SIMPLIFY THE LOOP, WE RUN DRAWS
OVER 1 TO 2*NDRAWS AND TREAT EVEN AND ODD VALUES DIFFERENTLY.
FOR ODD VALUES OF DRAWS, DRAW A NEW SWISH AND BETAU. FOR EVEN VALUES, USE THE PREVIOUS BETAU AND SUBTRACT RATHER THAN ADD.
*/
DO DRAWS = 1,NDRAWS*2
   IF %CLOCK(DRAWS,2)==1 {
      COMPUTE SIGMAD  =%MQFORM(%NOBS*INV(%RANWISHART(NVAR,%NOBS)),SVTR)
      COMPUTE SWISH   =%DECOMP(SIGMAD)
      COMPUTE RANC    =%RAN(1.0)
      COMPUTE BETAU   =SXX*RANC*TR(SWISH)
      COMPUTE BETADRAW=BETAOLS+BETAU


   }
   ELSE
   COMPUTE BETADRAW=BETAOLS-BETAU
   COMPUTE %MODELSETCOEFFS(VARMODEL,BETADRAW)
   IMPULSE(NOPRINT,MODEL=VARMODEL,DECOMP=SWISH,RESULTS=IMPULSES) NVAR NSTEP
*******RATS***********************************









*******RATS***********************************
/*
   FOR ODD VALUES, STORE THE IMPULSE RESPONSES. FOR EVEN VALUES, COMBINE THE STORED RESPONSES FROM THE PREVIOUS DRAW WITH THE
   RESPONSES FROM THE ANTITHETIC DRAW TO FORM THE H* FUNCTION FOR THE SQUARED IMPULSE RESPONSE. ACCUMULATE IT AND ITS SQUARE.
*/
   DO I=1,NVAR
      DO J=1,NVAR
         DO K=1,NSTEP
            COMPUTE FIRST(I,J)(K)=FIRST(I,J)(K)+.5*IMPULSES(I,J)(K)
            IF %CLOCK(DRAWS,2)==1
               COMPUTE HSTAR(I,J)(K)=IMPULSES(I,J)(K)
            ELSE {
               COMPUTE HSTAR(I,J)(K)=.5*(HSTAR(I,J)(K)**2+IMPULSES(I,J)(K)**2)
               COMPUTE SECOND(I,J)(K)=SECOND(I,J)(K)+HSTAR(I,J)(K)
               COMPUTE FOURTH(I,J)(K)=FOURTH(I,J)(K)+HSTAR(I,J)(K)**2
            }
         END DO K
      END DO J
   END DO I
END DO DRAWS

DO I=1,NVAR
   DO J=1,NVAR
      DO K=1,NSTEP
         COMPUTE FIRST(I,J)(K)=FIRST(I,J)(K)/NDRAWS
         COMPUTE SECOND(I,J)(K)=SECOND(I,J)(K)/NDRAWS
         COMPUTE FOURTH(I,J)(K)=FOURTH(I,J)(K)/NDRAWS-SECOND(I,J)(K)**2
         COMPUTE SECOND(I,J)(K)=SECOND(I,J)(K)-FIRST(I,J)(K)**2
      END DO K
   END DO J
END DO I
/*
THE FOLLOWING PRESENTS THE IMPULSE RESPONSES WITH UPPER AND LOWER TWO STANDARD ERROR BOUNDS. LOOKING DOWN A COLUMN, YOU SEE
THE RESPONSES OF A PARTICULAR VARIABLE. TO ENSURE THAT YOU DON'T INFLATE SMALL RESPONSES, THIS TAKES ONE PASS THROUGH A COLUMN TO
DD STANDARDIZES THE GRAPHS IN THE COLUMN ON THOSE.
*/

DEC VECT[STRINGS] XLABEL(NVAR) YLABEL(NVAR)
DEC VECT[INTEGER] DEPVARS
COMPUTE DEPVARS=%MODELDEPVARS(VARMODEL)
DO I=1,NVAR
   COMPUTE LL=%L(DEPVARS(I))
   COMPUTE XLABEL(I)='RESPONSE OF '+LL
   COMPUTE YLABEL(I)=LL
END DO I

GRPARM(BOLD) HLABEL 18 MATRIXLABELS 14
GRPARM  AXISLABEL 40 HEADER 40
SPGRAPH(XPOS=BOTH,XLAB=XLABEL, $
        YLAB=YLABEL,VLAB='SHOCK TO',VFIELDS=NVAR,HFIELDS=NVAR)
SMPL 1 NSTEP
DO I=1,NVAR
/*
DETERMINE THE RANGE NEEDED FOR ALL RESPONSES OF VARIABLE I. CALCULATE FROM THE LOWEST 2 S.E. LOWER BOUND TO THE HIGHEST 2 S.E UPPER BOUND.
*/
   COMPUTE MINLOWER=MAXUPPER=0.0
   DO J=1,NVAR
      SET RESP   = FIRST(I,J)(T)
      SET STDERR = SQRT(SECOND(I,J)(T))
      SET UPPER  = RESP + 1.96 * STDERR
      SET LOWER  = RESP - 1.96 * STDERR
      COMPUTE MAXUPPER=%MAX(MAXUPPER,%MAXVALUE(UPPER))
      COMPUTE MINLOWER=%MIN(MINLOWER,%MINVALUE(LOWER))
   END DO J
*
   DO J=1,NVAR
      SET RESP   = FIRST(I,J)(T)
      SET STDERR = SQRT(SECOND(I,J)(T))
      SET UPPER  = RESP + 1.96 * STDERR
      SET LOWER  = RESP - 1.96 * STDERR
      GRAPH(TICKS,PATTERN,MIN=MINLOWER,MAX=MAXUPPER,NUMBER=0) 3 I J
      # RESP
      # UPPER / 4
      # LOWER / 4
   END DO J
END DO I
SPGRAPH(DONE)
*******RATS***********************************







/*

********************* MATTEO **************************
*
* THIS PART OF THE CODE SAVES THE IMPULSE RESPONSES AND
* THEIR UPPER AND LOWER BOUNDS FROM THE VAR IN A TEXT FILE
* FOR USE IN THE ESTIMATION IN THE MODEL
* THE FILES RE, REU AND REL CONTAIN ALL THE VAR IMPULSE RESPONSES
* AND ARE USED BY THE
*
* VERY IMPORTANT: THE SECTION OF CODE BELOW IS COMMENTED, SO THAT
* THE UPPER AND LOWER CONFIDENCE INTERVAL DO NOT CHANGE
* EVERYTIME I RUN THE VAR
* UNCOMMENT THE LINES BELOW TO REPLACE OLD
* IMPULSE RESPONSES WITH SOME NEW ONES.
*
*******************************************************


DECLARE  RECT[SERIES] RE(NVAR,NVAR)
DECLARE  RECT[SERIES] REU(NVAR,NVAR)
DECLARE  RECT[SERIES] REL(NVAR,NVAR)

SMPL 1 NSTEP
DO I=1,NVAR
   DO J=1,NVAR
      SET RE(I,J)  = FIRST(I,J)(T)
      SET STDERR = SQRT(SECOND(I,J)(T))
      SET REU(I,J)  = RE(I,J) + 1.645 * STDERR
      SET REL(I,J)  = RE(I,J) - 1.645 * STDERR
   END DO J
END DO I


OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\RE.TX
COPY(ORG=OBS,FOR='(16F12.6)') / RE
CLOSE COPY


OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\REU.TX
COPY(ORG=OBS,FOR='(16F12.6)') / REU
CLOSE COPY


OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\REL.TX
COPY(ORG=OBS,FOR='(16F12.6)') / REL
CLOSE COPY


*********************** MATTEO **************************
*/


