
*****************************************
*
* SET UP AND ESTIMATION OF CONSUMPTION VAR IN THE PAPER
* "HOUSE PRICES, BORROWING CONSTRAINTS AND MONETARY POLICY IN THE BUSINESS CYCLE"
* BY MATTEO IACOVIELLO
*
* THE CONSUMPTION VAR IS USED TO COMPARING THE MODEL IMPULSE RESPONSES
* OF CONSUMPTION TO A HOUSE PRICE SHOCK TO THEIR EMPIRICAL COUNTERPARTS
* SEE FIGURE 3 IN THE PAPER AND THE SUBSECTION
* "COLLATERAL EFFECTS AND THE EFFECTS ON CONSUMPTION OF A HOUSE PRICE SHOCK"
*
******************************************************


**************************************************************************
*
* DOWNLOADING DATA AND STORING THEM
* THE DATA ARE CONSTRUCTED AND DESCRIBED IN THE EXCEL FILES IN THE FOLDER
* AND IN THE FILE BENCHMARK_VAR.PRG
*
**************************************************************************

CALENDAR 1969 1 4; ALL 2003:4 ;
OPEN DATA C:\E\HOUSEPRICES_AER\VAR_AER\USDATA_ALL.RAT; DATA(FOR=RAT, ORG=OBS)






**************************************************
* SPECIFICATION AND ESTIMATION OF CONSUMPTION VAR
**************************************************


COMPUTE NVAR=5			;*NUMBER OF VARIABLES
COMPUTE NSTEPS=22		;*NUMBER OF RESPONSE STEPS
COMPUTE NDRAWS=500		;*NUMBER OF DRAWS

* MATTEO, CORRECTION FOR NUMBER OF STEPS BASED ON TIME PERIOD
COMPUTE NSTEP = NSTEPS - 1;


COMPUTE BEG = 1974:1
COMPUTE ENDE = 2003:2

SMPL BEG ENDE
COMPUTE BEG_ = BEG
COMPUTE END_ = ENDE
COMPUTE LAGS = 2
SMPL BEG_ END_



SYSTEM(MODEL=VARMODEL)
VARIABLES R Q CE Y P
LAGS 1 TO LAGS
KFSET XXX
DET CONSTANT TIME LCRB{1} D794AFTER
END(SYSTEM)
ESTIMATE(OUTSIGMA=VMAT)




*************************************************************
*
* INDICATE WHICH SHOCK YOU ARE INTERESTED IN
* I JUST PLOT THE IRF TO A SHOCK IN THE SECOND VARIABLE (Q)
*
*************************************************************

COMPUTE SHOCKINDEX = 2







*******************************************************
* THE STUFF BELOW IS TAKEN FROM THE MONTEVAR.PRG FILE
* AVAILABLE FROM RATS PUBLIC DOMAIN
* SOME SMALL CHANGES CAN BE FOUND SEARCHING FOR THE WORD "MATTEO"
********************************************************


DECLARE RECT SXX SVTR SWISH BETAOLS BETADRAW
DECLARE SYMM SIGMAD

COMPUTE SXX    =%DECOMP(XXX)
COMPUTE SVTR   =TR(%DECOMP(VMAT))
COMPUTE BETAOLS=%MODELGETCOEFFS(VARMODEL)
COMPUTE NCOEF  =%ROWS(SXX)
COMPUTE WISHDOF=%NOBS-NCOEF

DECLARE RECT RANC(NCOEF,NVAR)

*
*  THIS CODE HANDLES THE ANTITHETIC DRAWS DIFFERENTLY THAN THE ORIGINAL MONTEVAR PROGRAM.
*  INSTEAD OF DOUBLING UP ON NDRAWS AND COMBINING RESULTS OF THE ORIGINAL DRAW WITH ITS
*  ANTITHETE, IT JUST DOES A NEW DRAW ON THE ODD VALUES, AND THE ANTITHETIC DRAW ON EVEN ONES.
*  EACH IS STORED SEPARATELY.
*
*  ON THE ODD VALUES FOR DRAWS, A DRAW IS MADE FROM THE INVERSE WISHART DISTRIBUTION FOR THE
*  COVARIANCE MATRIX. THIS ASSUMES USE OF THE JEFFREY'S PRIOR |S|**-(N+1)/2 WHERE N IS THE
*  NUMBER OF EQUATIONS IN THE VAR. THE POSTERIOR FOR S WITH THAT PRIOR IS INV(S)~WISHART WITH
*  T-P D.F. (P = NUMBER OF EXPLANATORY VARIABLES PER EQUATION) AND COVARIANCE MATRIX INV(T(S-HAT)).
*
*  A DRAW FOR THE COEFFICIENTS IS COMPUTED BY POSTMULTIPLYING THE KRONECKER PRODUCT OF THE DECOMPOSITIONS
*  OF THE SIGMA DRAW AND X'X INVERSE BY A VECTOR OF STANDARD NORMAL DRAWS. FOR EVEN VALUES OF "DRAWS",
*  WE TAKE THE REFLECTION OF THE ORIGINAL DRAW AROUND THE OLS VECTOR.
*
*
DECLARE VECT[RECT] RESPONSES(NDRAWS)
DECLARE RECT[SERIES] IMPULSES(NVAR,NVAR)

* MATTEO
DECLARE RECT[SERIES] IRF(NVAR,NVAR)
DECLARE RECT[SERIES] IRFUP(NVAR,NVAR)
DECLARE RECT[SERIES] IRFLO(NVAR,NVAR)





INFOBOX(ACTION=DEFINE,PROGRESS,LOWER=1,UPPER=NDRAWS) 'MONTE CARLO INTEGRATION'
DO DRAWS = 1,NDRAWS
   IF %CLOCK(DRAWS,2)==1 {
      COMPUTE SIGMAD  =%MQFORM(%NOBS*INV(%RANWISHART(NVAR,WISHDOF)),SVTR)
      COMPUTE SWISH   =%DECOMP(SIGMAD)
      COMPUTE RANC    =%RAN(1.0)
      COMPUTE BETAU   =SXX*RANC*TR(SWISH)
      COMPUTE BETADRAW=BETAOLS+BETAU
   }
   ELSE
      COMPUTE BETADRAW=BETAOLS-BETAU
   COMPUTE %MODELSETCOEFFS(VARMODEL,BETADRAW)
   IMPULSE(NOPRINT,MODEL=VARMODEL,DECOMP=SWISH,RESULTS=IMPULSES) NVAR NSTEP
*
*     STORE THE IMPULSE RESPONSES
*
   DIM RESPONSES(DRAWS)(NVAR*NVAR,NSTEP)
   EWISE RESPONSES(DRAWS)(I,J)=IMPULSES((I-1)/NVAR+1,%CLOCK(I,NVAR))(J)
   INFOBOX(CURRENT=DRAWS)
END DO DRAWS

DEC VECT[STRINGS] XLABEL(NVAR) YLABEL(NVAR)
DEC VECT[INTEGER] DEPVARS
COMPUTE DEPVARS=%MODELDEPVARS(VARMODEL)
DO I=1,NVAR
   COMPUTE LL=%L(DEPVARS(I))
   COMPUTE XLABEL(I)=LL
   COMPUTE YLABEL(I)=LL
END DO I
INFOBOX(ACTION=REMOVE)


/*
GRPARM HEADER 20 HLABEL 18 MATRIXLABELS 16 AXISLABEL 55
SPGRAPH(HEADER='IMPULSE RESPONSES',XPOS=BOTH,XLAB=XLABEL, $
        YLAB=YLABEL,VLAB='SHOCK TO',VFIELDS=NVAR,HFIELDS=NVAR) ;  * MATTEO CHANGE RESPONSES OF TO SHOCK TO
*/


*
*  BECAUSE WE WANT A COMMON SCALE FOR ALL RESPONSES OF A SINGLE VARIABLE,
*  WE NEED TO DO ALL THE CALCULATIONS FOR A FULL ROW OF GRAPHS FIRST.
*

DEC VECT[SERIES] UPPERI(NVAR) LOWERI(NVAR) RESPI(NVAR)
DO I=1,NVAR
   COMPUTE MINLOWER=MAXUPPER=0.0
   SMPL 1 NDRAWS
   DO J=1,NVAR
      CLEAR LOWERI(J) UPPERI(J) RESPI(J)
      DO K=1,NSTEP
         SET WORK   1 NDRAWS = RESPONSES(T)((I-1)*NVAR+J,K)
         COMPUTE FRAC=%FRACTILES(WORK,||.05,.95||)
         COMPUTE LOWERI(J)(K)=FRAC(1)
         COMPUTE UPPERI(J)(K)=FRAC(2)
         COMPUTE RESPI(J)(K)=%AVG(WORK)
      END DO K
      COMPUTE MAXUPPER=%MAX(MAXUPPER,%MAXVALUE(UPPERI(J)))
      COMPUTE MINLOWER=%MIN(MINLOWER,%MINVALUE(LOWERI(J)))
   END DO J
*
   SMPL 1 NSTEP
   DO J=1,NVAR

   SET IRF(I,J) = RESPI(J)
   SET IRFUP(I,J) = UPPERI(J)
   SET IRFLO(I,J) = LOWERI(J)



   END DO J   ;
END DO I













********************************************************************************
*
* A FEW LINES OF PROGRAM ADDED IN ORDER TO PLOT ONLY THE HOUSE PRICE SHOCK
*
********************************************************************************

GRPARM HEADER 20

SMPL 1 NSTEP

     SPGRAPH(VFIELDS=3,HFIELDS=2,HEADER='EFFECTS OF A HOUSE PRICE SHOCK - 1 STD')

GRPARM HLABEL 18 MATRIXLABELS 16 AXISLABEL 42 HEADER 40

    DO I=1,NVAR


       GRAPH(HEADER=YLABEL(I),TICKS,PATTERN,NUMBER=0) 3
       # IRF(I,SHOCKINDEX)
       # IRFUP(I,SHOCKINDEX) / 4
       # IRFLO(I,SHOCKINDEX) / 4
    END DO I


   SPGRAPH(DONE)











****************************************************
*
* THE LINES BELOW SAVE THE IMPULSE RESPONSES
* IN THE FILES Q_SHOCK, Q_SHOCK_UP AND Q_SHOCK_LO
*
*****************************************************




/*


OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\Q_SHOCK.TX
COPY(ORG=OBS,FOR='(16F12.6)') / IRF(2,2) IRF(3,2)
CLOSE COPY

OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\Q_SHOCK_UP.TX
COPY(ORG=OBS,FOR='(16F12.6)') / IRFUP(2,2) IRFUP(3,2)
CLOSE COPY

OPEN COPY C:\E\HOUSEPRICES_AER\VAR_AER\Q_SHOCK_LO.TX
COPY(ORG=OBS,FOR='(16F12.6)') / IRFLO(2,2) IRFLO(3,2)
CLOSE COPY


*/

